;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Little Hedgehog
; By: Schwa
;
; What this does:
; This enemy slowly walks along the ground, and when it sees Mario he stays still and extends
; his spines, rendering him immune to jump attacks (spin-jumps will make Mario bounce off).
; It will not extend its spines if Mario is behind the sprite, so Mario can defeat it by coming
; up from behind and jumping on it.
;
; Extra Bit:
; This sprite does not use the Extra Bit.
;
; Customizability:
; The two Range variables below can be changed to set the max and min distance Mario needs to
; be in for the sprite to extend its spines.
;
; Changing the First byte in the CFG file will have no effect on the sprite. That value is
; coded into the ASM file and is reset every frame.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	RANGE_1 = $3A	; This is the maximum distance Mario can be to trigger Defense Mode.
	RANGE_2 = $01	; This is the minimum distance.

;;;;;;;;;;;;;;;;
; Init Routine ;
;;;;;;;;;;;;;;;;

	dcb "INIT"
        LDA #$05
        STA $7F8888
	JSR SUB_HORZ_POS	; \
	TYA			;  | face Mario initially
	STA $157C,x		; /
	RTL

;;;;;;;;;;;;;;;;
; Main Routine ;
;;;;;;;;;;;;;;;;

	dcb "MAIN"
	PHB
	PHK
	PLB
        JSR SPRITE_ROUTINE      ; Aka health routine.	
        LDA $157C,x             ; $02 = sprite direction
        STA $02                     
	JSR DEC_TIMERS
	JSR MAIN_CODE
	PLB
	RTL

DEC_TIMERS:
	LDA $C2,x
	BEQ NO_DEC
	DEC $C2,x
NO_DEC:
	RTS

;;;;;;;;;;;;;;;;;;;;
; Main Sprite Code ;
;;;;;;;;;;;;;;;;;;;;

X_SPEED:
	dcb $08,$F8

KILLED_1:
	LDA #$80	; \ when killed, fall through all objects.
	STA $1686,x	; /
RETURN1:
	RTS
MAIN_CODE:
	JSR PROXIMITY	; \
	BEQ NO_RANGE	;  | if Mario is in the sprite's detection range, go into Defense Mode
	LDA #$40	;  |
	STA $C2,x	; /
NO_RANGE:
	LDA $C2,x		; \ if in Defense Mode, call different sprite code
	BNE DEFENSE_MODE_CODE	; /

	JSR SUB_GFX	; call graphics routine

	LDA $14C8,x		; \
	CMP #$08		;  | don't process if sprites locked or if killed
	BNE KILLED_1		;  |
	LDA $9D			;  |
	BNE RETURN1		; /
	JSR SUB_OFF_SCREEN_X0	; handle when off screen

	LDA $1588,x	; \
	AND #$04	;  | set X speed based on direction, as long as sprite is on the ground
	BEQ FINISH	;  |
	LDY $157C,x	;  |
	LDA X_SPEED,y	;  |
	STA $B6,x	; /

FINISH:
	BRA CHECK_CONTACT

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; "Defense Mode" Sprite Code ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DEFENSE_MODE_CODE:
	STZ $1656,x	; set CFG properties to make the sprite invincible

	JSR SUB_GFX_DEFENSE	; call graphics routine, which is different in Defense Mode

	LDA $14C8,x		; \
	CMP #$08		;  | don't process if sprites locked or if killed
	BNE KILLED_1		;  |
	LDA $9D			;  |
	BNE RETURN3		; /
	JSR SUB_OFF_SCREEN_X0	; handle when off screen
	
	STZ $B6,x	; in Defense Mode, the sprite doesn't move

	JSR SUB_HORZ_POS	; \
	TYA			;  | face Mario
	STA $157C,x		; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end for both above routines ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CHECK_CONTACT:
	JSL $01802A	; factor in X and Y speed
	LDA $1588,x	; check to see if sprite is touching an object
        AND #$03
        BEQ NO_OBJ_CONTACT
	LDA $157C,x	; \
	EOR #$01	;  | if so, flip the horizontal direction
	STA $157C,x	; /
NO_OBJ_CONTACT:
	JSL $018032	; interact with sprites
	JSL $01A7DC	; check for Mario-Sprite contact

RETURN3:
	RTS
;;;;;;;;;;;;;;;;;;;;;
; hitpoint routine? ;
;;;;;;;;;;;;;;;;;;;;;
SPRITE_ROUTINE      LDA $C2,x               ; If he's defending...
                    CMP #$40
                    BEQ NOT_HURT            ; Don't let the block take his HP away.
                    LDY #$0B                ; Also just code from the newbieboss...
LOOP		    LDA $14C8,y             ; \ if the sprite status is..
		    CMP #$09                ;  | ...shell-like
		    BEQ PROCESS_SPRITE      ; /
		    CMP #$0A                ; \ ...throwned shell-like
		    BEQ PROCESS_SPRITE      ; /
NEXT_SPRITE	    DEY                     ;
		    BPL LOOP                ; ...otherwise, loop
		    RTS                     ; return

PROCESS_SPRITE	    PHX                     ; push x
		    TYX                     ; transfer x to y
		    JSL $03B6E5             ; get sprite clipping B routine
		    PLX                     ; pull x
		    JSL $03B69F             ; get sprite clipping A routine
		    JSL $03B72B             ; check for contact routine
		    BCC NEXT_SPRITE         ;

		    LDA #$03                ; \ set sprite state
		    STA $C2,x               ; /
		    LDA #$40                ; \ set timer
		    STA $1540,x             ; /

		    PHX                     ; push x
		    TYX                     ; transfer x to y

		    STZ $14C8,x             ; destroy the sprite

BLOCK_SETUP	    LDA $E4,x               ; \ setup block properties
                    STA $9A                 ;  |
                    LDA $14E0,x             ;  |
                    STA $9B                 ;  |
                    LDA $D8,x               ;  |
                    STA $98                 ;  |
                    LDA $14D4,x             ;  |
                    STA $99                 ; /

EXPLODING_BLOCK	    PHB                     ; \ set the exploding block routine
		    LDA #$02                ;  |
		    PHA                     ;  |
		    PLB                     ;  |
		    LDA #$FF                ;  | $FF = set flashing palette
		    JSL $028663             ;  |
                    LDA $7F8888
                    DEC A
                    STA $7F8888
                    PLB                     ; /

		    PLX                     ; pull x
		    LDA #$28                ; \ sound effect
		    STA $1DFC               ; /
                    BRA CHECK
                    
NOT_HURT            LDY #$0B                ; Also just code from the newbieboss...
LOOP2		    LDA $14C8,y             ; \ if the sprite status is..
		    CMP #$09                ;  | ...shell-like
		    BEQ PROCESS_SPRITE2     ; /
		    CMP #$0A                ; \ ...throwned shell-like
		    BEQ PROCESS_SPRITE2     ; /
NEXT_SPRITE2	    DEY                     ;
		    BPL LOOP2               ; ...otherwise, loop
		    RTS                     ; return

PROCESS_SPRITE2	    PHX                     ; push x
		    TYX                     ; transfer x to y
		    JSL $03B6E5             ; get sprite clipping B routine
		    PLX                     ; pull x
		    JSL $03B69F             ; get sprite clipping A routine
		    JSL $03B72B             ; check for contact routine
		    BCC NEXT_SPRITE2        ;

		    LDA #$03                ; \ set sprite state
		    STA $C2,x               ; /
		    LDA #$40                ; \ set timer
		    STA $1540,x             ; /

		    PHX                     ; push x
		    TYX                     ; transfer x to y

		    STZ $14C8,x             ; destroy the sprite

BLOCK_SETUP2	    LDA $E4,x               ; \ setup block properties
                    STA $9A                 ;  |
                    LDA $14E0,x             ;  |
                    STA $9B                 ;  |
                    LDA $D8,x               ;  |
                    STA $98                 ;  |
                    LDA $14D4,x             ;  |
                    STA $99                 ; /

EXPLODING_BLOCK2    PHB                     ; \ set the exploding block routine
		    LDA #$02                ;  |
		    PHA                     ;  |
		    PLB                     ;  |
		    LDA #$FF                ;  | $FF = set flashing palette
		    JSL $028663             ;  |
                    PLB
                    PLX                     ; pull x

CHECK               LDA $7F8888
                    CMP #$00
                    BEQ DEAD
                    RTS
DEAD                LDA #$04
                    STA $14C8,x
                    LDA #$FF
                    STA $1493
                    LDA #$0B
                    STA $1DFB
                    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Defense Mode Graphics Routine ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TILEMAP2:
	dcb $84,$84,$80		; Walking 1, walking 2, killed.
HORZ_DISP:
	dcb $00,$01,$00

SUB_GFX_DEFENSE:
	JSR GET_DRAW_INFO	; We still need this. We always will.

	LDA $157C,x	; \ store the sprite direction so we don't have to screw up the X Register
	STA $02		; /

        LDA $14C8,x		; If killed...
        CMP #$02
	BNE NOTKILLED_2
	LDA #$02                ;    ...set killed frame
        STA $03			
        LDA $15F6,x		;    ...flip vertically
        ORA #$80
        STA $15F6,x
	BRA DRAW_2

NOTKILLED_2:
	LDA $14		; \
	LSR A		;  | Just one LSR. This will animate the sprite SUPER fast.
	CLC		;  |
	ADC $15E9	;  |
	AND #$01	;  | Change this AND command to higher values to give the sprite multiple animation frames.
	STA $03		; / 

DRAW_2:
	PHX		; Preserve X so we can manhandle it. Muhuhuhu...
	PHX

	LDX $03		; We need to set X with the animation frame in order to work the table.
	LDA $00		; Tile x position = sprite x location ($00)
	CLC		; \ Unlike before, instead of the Tilemap being animated, it's the horizontal positioning.
	ADC HORZ_DISP,x	; / Given the values of the Table above, this means the sprite will tremble!
	STA $0300,y	; 

	LDA $01		; Tile y position = sprite y location ($01)
	STA $0301,y	; Not changing Y though. ^_^

	PLX
	LDA $15F6,x	; Tile properties xyppccct, format
	ORA $64		; Add in tile priority of level...
	STA $0303,y	; ...and write everything to $0303, 'cause that's what it's there for.

	LDX $03		; \
	LDA TILEMAP2,x	;  | Whatever the animation frame is, this code will recall it.
	STA $0302,y	; /
	PLX		; Restore X a second time. No wonder we pushed it twice.

	LDY #$02	; The tiles are 16x16...
	LDA #$00	; ...and we drew one of them...
	JSL $01B7B3	; With this info, we can call the Bookkeeping routine...

	RTS		; ...and that's it for the Defense Mode graphics!

;;;;;;;;;;;;;;;;;;;;
; Graphics Routine ;
;;;;;;;;;;;;;;;;;;;;

TILEMAP             dcb $80,$82,$A4,$A6,$C0,$C2,$E0,$E2
                    dcb $80,$82,$A4,$A6,$C0,$C2,$E0,$E2
HORIZ_DISP          dcb $F8,$08,$F8,$08,$F8,$08,$F8,$08
                    dcb $08,$F8,$08,$F8,$08,$F8,$08,$F8
VERT_DISP           dcb $F0,$F0,$00,$00,$F0,$F0,$00,$00
                    dcb $F0,$F0,$00,$00,$F0,$F0,$00,$00
PROPERTIES          dcb $40,$00             ;xyppccct format

SUB_GFX             LDA #$96		; \ adjust sprite clippings
                    STA $1662,x		; /
                    LDA #$00		; \ disallow sprite to be jumped on
                    STA $1656,x		; /
                    LDA #$01		; \ Yoshi can not eat the sprite
                    STA $1686,x		; /

                    JSR GET_DRAW_INFO       ; after: Y = index to sprite tile map ($300)
                                            ;      $00 = sprite x position relative to screen boarder
                                            ;      $01 = sprite y position relative to screen boarder
                    LDA $1602,x
                    STA $03                 ; | $03 = index to frame start (0 or 4)                   
                   
                    LDA $157C,x             ; \ $02 = sprite direction
                    STA $02                 ; /
                    BNE NO_ADD
                    LDA $03
                    CLC
                    ADC #$08
                    STA $03

NO_ADD              LDA $14	; \
                    LSR A	;  | this code animates the sprite
                    LSR A	;  |
                    CLC		;  | more "LSR A" commands mean slower animation
                    ADC $15E9	;  |
                    AND #$01	;  |
                    BEQ NO_ADD2 ;  |
                    LDA $03     ;  |
                    CLC         ;  |
                    ADC #$04    ;  |
                    STA $03	; /

NO_ADD2             PHX                     ; push sprite index

                    LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START_2        PHX                     ; push current tile number
                    TXA                     ; \ X = index to horizontal displacement
                    ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT         TAX                     ; \ 
                    
                    LDA $00                 ; \ tile x position = sprite x location ($00)
                    CLC                     ;  |
                    ADC HORIZ_DISP,x        ;  |
                    STA $0300,y             ; /
                    
                    LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC VERT_DISP,x         ;  |
                    STA $0301,y             ; /
                    
                    LDA TILEMAP,x           ; \ store tile
                    STA $0302,y             ; / 
        
                    LDX $02                 ; \
                    LDA PROPERTIES,x        ;  | get tile properties using sprite direction
                    LDX $15E9               ;  |
                    ORA $15F6,x             ;  | get palette info
                    ORA $64                 ;  | put in level properties
                    STA $0303,y             ; / store tile properties
                    
                    PLX                     ; \ pull, X = current tile of the frame we're drawing
                    INY                     ;  | increase index to sprite tile map ($300)...
                    INY                     ;  |    ...we wrote 1 16x16 tile...
                    INY                     ;  |    ...sprite OAM is 8x8...
                    INY                     ;  |    ...so increment 4 times
                    DEX                     ;  | go to next tile of frame and loop
                    BPL LOOP_START_2        ; / 

                    PLX                     ; pull, X = sprite index
                    LDY #$02                ; \ 02, because we didn't write to 460 yet
                    LDA #$03                ;  | A = number of tiles drawn - 1
                    JSL $01B7B3             ; / don't draw if offscreen
                    RTS                     ; return


;;;;;;;;;;;;;;;;;;;;;
; Proximity Routine ;
;;;;;;;;;;;;;;;;;;;;;

EORTBLI	dcb $FF,$00
		
PROXIMITY
	LDA $14C8,x
	CMP #$08
	BNE PRange_Out

	JSR SUB_HORZ_POS
	TYA
	CMP $157C,x
	BNE PRange_Out
	
	LDA $14E0,x		;sprite x high
	XBA
	LDA $E4,x		;sprite x low
	REP #$20		;16bitA
	SEC
	SBC $94			;sub mario x
	SEP #$20		;8bitA
	PHA			;preserve for routine jump
	JSR SUB_HORZ_POS	;horizontal distance
	PLA			;restore
	EOR EORTBLI,y		;invert if needed
	CMP #RANGE_1
	BCS PRange_Out		;return not within range
	CMP #RANGE_2
	BCC PRange_Out
	LDA #$01		;Z = 0
	RTS

PRange_Out
	LDA #$00		;Z = 1
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine - unknown
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  } 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give mario points
                    PLY                     ;
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642
